@@ -1,8 +1,8 @@
-#!/usr/local/bin/perl -w
+#!/usr/local/bin/perl
 
 #---------------------------------------------------------
 #
-# $Id: test.pl,v 1.10 1997/10/05 18:25:56 mergl Exp $
+# $Id: test.pl,v 1.12 1998/02/01 18:40:35 mergl Exp $
 #
 # Portions Copyright (c) 1994,1995,1996,1997 Tim Bunce
 # Portions Copyright (c) 1997                Edmund Mergl
@@ -14,7 +14,7 @@
 
 ######################### We start with some black magic to print on failure.
 
-BEGIN { $| = 1; print "1..33\n"; }
+BEGIN { $| = 1; print "1..26\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use DBI;
 $loaded = 1;
@@ -25,16 +25,19 @@ $| = 1;
 ######################### End of black magic.
 
 # supply userid and password below, if access to 
-# your databases is protected in data/pg_hba.conf.
+# your databases is protected in pgsql/data/pg_hba.conf.
 
 $dbmain = 'template1';
 $dbname = 'pgperltest';
+$dbhost = 'localhost';
 $dbuser = '';
 $dbpass = '';
 
+# DBI->trace(1); # make your choice
+
 ######################### create test database
 
-$dbh = DBI->connect("dbi:Pg:$dbmain", $dbuser, $dbpass) or die $DBI::errstr;
+$dbh = DBI->connect("dbi:Pg:dbname=$dbmain;host=$dbhost", $dbuser, $dbpass, {PrintError => 0}) or die $DBI::errstr;
 
 $dbh->do("DROP DATABASE $dbname");
 $dbh->do("CREATE DATABASE $dbname");
@@ -43,27 +46,31 @@ $dbh->disconnect;
 
 ######################### create, insert, update, delete, drop
 
-( $dbh = DBI->connect("dbi:Pg:$dbname", $dbuser, $dbpass) )
+# connect to database and create table
+
+( $dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$dbhost", $dbuser, $dbpass) )
     and print "ok 2\n"
     or  die $DBI::errstr;
 
 ( $dbh->do( "CREATE TABLE builtin ( 
-  bool    bool,
-  char    char,
-  char16  char16,
-  text    text,
-  date    date,
-  int4    int4,
-  int4_   int4[],
-  float8  float8,
-  point   point,
-  lseg    lseg,
-  box     box
+  bool_    bool,
+  char_    char,
+  char16_  char16,
+  text_    text,
+  date_    date,
+  int4_    int4,
+  int4a_   int4[],
+  float8_  float8,
+  point_   point,
+  lseg_    lseg,
+  box_     box
   )" ) )
     and print "ok 3\n"
     or  die $DBI::errstr;
 
-( $dbh->do( "INSERT INTO builtin VALUES(
+# insert into table with $dbh->do(), and then using placeholders
+
+( 1 == $dbh->do( "INSERT INTO builtin VALUES(
   't',
   'a',
   'Edmund Mergl',
@@ -79,7 +86,14 @@ $dbh->disconnect;
     and print "ok 4\n"
     or  die $DBI::errstr;
 
-( $dbh->do( "INSERT INTO builtin VALUES( 
+( $sth = $dbh->prepare( "INSERT INTO builtin 
+  ( bool_, char_, char16_, text_, date_, int4_, int4a_, float8_, point_, lseg_, box_ )
+  VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )
+  " ) )
+    and print "ok 5\n"
+    or  die $DBI::errstr;
+
+( $sth->execute (
   'f',
   'b',
   'Halli Hallo',
@@ -91,11 +105,11 @@ $dbh->disconnect;
   '(4.0,5.0)',
   '((4.0,5.0),(6.0,7.0))',
   '((4.0,5.0),(6.0,7.0))'
-  )" ) )
-    and print "ok 5\n"
-    or  die $DBI::errstr;
+  ) )
+    and print "ok 6\n"
+    or die $DBI::errstr;
 
-( $sth = $dbh->prepare( "INSERT INTO builtin VALUES( 
+( $sth->execute (
   'f',
   'c',
   'Potz Blitz',
@@ -107,13 +121,11 @@ $dbh->disconnect;
   '(2.0,7.0)',
   '((2.0,7.0),(8.0,3.0))',
   '((2.0,7.0),(8.0,3.0))'
-  )" ) )
-    and print "ok 6\n"
+  ) )
+    and print "ok 7\n"
     or  die $DBI::errstr;
 
-( $sth->execute )
-    and print "ok 7\n"
-    or die $DBI::errstr;
+# test pgsql-specific stuff
 
 $oid_status = $sth->{'pg_oid_status'};
 ( $oid_status ne '' )
@@ -129,109 +141,87 @@ $cmd_status = $sth->{'pg_cmd_status'};
     and print "ok 10\n"
     or  die $DBI::errstr;
 
-( $sth = $dbh->prepare( "SELECT * FROM builtin" ) )
+# select from table using input parameters and test various fetch methods
+
+( $sth = $dbh->prepare( "SELECT * FROM builtin where int4_ < ?" ) )
     and print "ok 11\n"
     or  die $DBI::errstr;
 
-( $sth->execute )
+( $sth->bind_param(1, 10000) )
     and print "ok 12\n"
     or  die $DBI::errstr;
 
+( $sth->execute )
+    and print "ok 13\n"
+    or  die $DBI::errstr;
+
 @row_ary = $sth->fetchrow_array;
 ( join(" ", @row_ary) eq '1 a Edmund Mergl Edmund Mergl 08-03-1997 1234 {1,2,3} 1.234 (1,2) [(1,2),(3,4)] (3,4),(1,2)' ) 
-    and print "ok 13\n"
-    or  print "not ok 13: row = ", join(" ", @row_ary), "\n";
+    and print "ok 14\n"
+    or  print "not ok 14: row = ", join(" ", @row_ary), "\n";
 
 $ary_ref = $sth->fetchrow_arrayref;
 ( join(" ", @$ary_ref) eq '0 b Halli Hallo Halli Hallo 06-01-1995 5678 {5,6,7} 5.678 (4,5) [(4,5),(6,7)] (6,7),(4,5)' )
-    and print "ok 14\n"
-    or  print "not ok 14: ary_ref = ", join(" ", @$ary_ref), "\n";
+    and print "ok 15\n"
+    or  print "not ok 15: ary_ref = ", join(" ", @$ary_ref), "\n";
 
 $hash_ref = $sth->fetchrow_hashref;
-( join(" ", (($key,$val) = each %$hash_ref)) eq 'text Potz Blitz' )
-    and print "ok 15\n"
-    or  print "not ok 15: key = $key, val = $val\n";
+( join(" ", (($key,$val) = each %$hash_ref)) eq 'text_ Potz Blitz' )
+    and print "ok 16\n"
+    or  print "not ok 16: key = $key, val = $val\n";
+
+# test various attributes
 
 @name = @{$sth->{'NAME'}};
-( join(" ", @name) eq 'bool char char16 text date int4 int4_ float8 point lseg box' )
-    and print "ok 16\n"
-    or  print "not ok 16: name = ", join(" ", @name), "\n";
+( join(" ", @name) eq 'bool_ char_ char16_ text_ date_ int4_ int4a_ float8_ point_ lseg_ box_' )
+    and print "ok 17\n"
+    or  print "not ok 17: name = ", join(" ", @name), "\n";
 
 @type = @{$sth->{'TYPE'}};
 ( join(" ", @type) eq '16 18 20 25 1082 23 1007 701 600 601 603' )
-    and print "ok 17\n"
-    or  print "not ok 17: type = ", join(" ", @type), "\n";
+    and print "ok 18\n"
+    or  print "not ok 18: type = ", join(" ", @type), "\n";
 
 @size = @{$sth->{'SIZE'}};
 ( join(" ", @size) eq '1 1 16 -1 4 4 -1 8 16 32 32' )
-    and print "ok 18\n"
-    or  print "not ok 18: size = ", join(" ", @size), "\n";
+    and print "ok 19\n"
+    or  print "not ok 19: size = ", join(" ", @size), "\n";
 
 print "not " if $sth->rows != 3;
-print "ok 19\n";
-
-print "not " if $DBI::rows != 3;
 print "ok 20\n";
 
+print "not " if $DBI::rows != 3;
+print "ok 21\n";
+
+# test binding of output columns
+
 ( $sth->execute )
-    and print "ok 21\n"
+    and print "ok 22\n"
+    or die $DBI::errstr;
+
+( $sth->bind_columns(undef, \$bool, \$char, \$char16, \$text, \$date, \$int4, \$int4a, \$float8, \$point, \$lseg, \$box) )
+    and print "ok 23\n"
     or die $DBI::errstr;
 
-$sth->bind_columns(undef, \$bool, \$char, \$char16, \$text, \$date, \$int4, \$int4_, \$float8, \$point, \$lseg, \$box);
 $sth->fetch;
-( "$bool, $char, $char16, $text, $date, $int4, $int4_, $float8, $point, $lseg, $box" eq 
+( "$bool, $char, $char16, $text, $date, $int4, $int4a, $float8, $point, $lseg, $box" eq 
   '1, a, Edmund Mergl, Edmund Mergl, 08-03-1997, 1234, {1,2,3}, 1.234, (1,2), [(1,2),(3,4)], (3,4),(1,2)' )
-    and print "ok 22\n"
-    or  print "not ok 22: $bool, $char, $char16, $text, $date, $int4, $int4_, $float8, $point, $lseg, $box\n";
+    and print "ok 24\n"
+    or  print "not ok 24: $bool, $char, $char16, $text, $date, $int4, $int4a, $float8, $point, $lseg, $box\n";
 
 ( $sth->finish )
-    and print "ok 23\n"
-    or  die $DBI::errstr;
-
-( $dbh->do( "UPDATE builtin SET int4 = 3 WHERE text = 'Edmund Mergl'" ) )
-    and print "ok 24\n"
-    or  die $DBI::errstr;
-
-( $sth = $dbh->prepare( "UPDATE builtin SET int4 = int4 + 1" ) )
     and print "ok 25\n"
     or  die $DBI::errstr;
 
-( 3 == $sth->execute )
-    and print "ok 26\n"
-    or  print "not ok 26\n";
-
-( $sth->finish )
-    and print "ok 27\n"
-    or die $DBI::errstr;
-
-( 1 == $dbh->do( "DELETE FROM builtin WHERE int4 = 4" ) )
-    and print "ok 28\n"
-    or  print "not ok 28\n";
-
-( $sth = $dbh->tables )
-    and print "ok 29\n"
-    or  die $DBI::errstr;
-
-$ary_ref = $sth->fetchrow_arrayref;
-( join(" ", @ary_ref) !~ /builtin r f$/ )
-    and print "ok 30\n"
-    or  print "not ok 30: ary_ref = ", @ary_ref, "\n";
-
-( $sth->finish )
-    and print "ok 31\n"
-    or  die $DBI::errstr;
-
-( $dbh->do( "DROP TABLE builtin" ) )
-    and print "ok 32\n"
-    or  die $DBI::errstr;
+# disconnect
 
 ( $dbh->disconnect )
-    and print "ok 33\n"
+    and print "ok 26\n"
     or  die $DBI::errstr;
 
 ######################### disconnect and drop test database
 
-$dbh = DBI->connect("dbi:Pg:$dbmain", $dbuser, $dbpass) or die $DBI::errstr;
+$dbh = DBI->connect("dbi:Pg:dbname=$dbmain;host=$dbhost", $dbuser, $dbpass) or die $DBI::errstr;
 
 $dbh->do("DROP DATABASE $dbname");
 
